searchData={"content_type":"text/markdown","items":[{"doc":"Nebulex adapter for Redis. This adapter is implemented using `Redix`,\na Redis driver for Elixir.\n\n**NebulexRedisAdapter** provides three setup alternatives:\n\n  * **Standalone** - The adapter establishes a pool of connections\n    with a single Redis node. The `:standalone` is the default mode.\n\n  * **Redis Cluster** - [Redis Cluster](https://redis.io/topics/cluster-tutorial)\n    is a built-in feature in Redis since version 3, and it may be the most\n    convenient and recommendable way to set up Redis in a cluster and have\n    a distributed cache storage out-of-box. This adapter provides the\n    `:redis_cluster` mode to set up **Redis Cluster** from the client-side\n    automatically and be able to use it transparently.\n\n  * **Built-in client-side cluster based on sharding** - This adapter\n    provides a simple client-side cluster implementation based on\n    Sharding distribution model via `:client_side_cluster` mode.","ref":"NebulexRedisAdapter.html","title":"NebulexRedisAdapter","type":"module"},{"doc":"We can define a cache to use Redis as follows:\n\n    defmodule MyApp.RedisCache do\n      use Nebulex.Cache,\n        otp_app: :nebulex,\n        adapter: NebulexRedisAdapter\n    end\n\nThe configuration for the cache must be in your application environment,\nusually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.RedisCache,\n      conn_opts: [\n        host: \"127.0.0.1\",\n        port: 6379\n      ]","ref":"NebulexRedisAdapter.html#module-standalone","title":"Standalone - NebulexRedisAdapter","type":"module"},{"doc":"We can define a cache to use Redis Cluster as follows:\n\n    defmodule MyApp.RedisClusterCache do\n      use Nebulex.Cache,\n        otp_app: :nebulex,\n        adapter: NebulexRedisAdapter\n    end\n\nThe config:\n\n    config :my_app, MyApp.RedisClusterCache,\n      mode: :redis_cluster,\n      redis_cluster: [\n        configuration_endpoints: [\n          endpoint1_conn_opts: [\n            host: \"127.0.0.1\",\n            port: 6379,\n            # Add the password if 'requirepass' is on\n            password: \"password\"\n          ],\n          ...\n        ]\n      ]","ref":"NebulexRedisAdapter.html#module-redis-cluster","title":"Redis Cluster - NebulexRedisAdapter","type":"module"},{"doc":"We can define a cache with \"client-side cluster mode\" as follows:\n\n    defmodule MyApp.ClusteredCache do\n      use Nebulex.Cache,\n        otp_app: :nebulex,\n        adapter: NebulexRedisAdapter\n    end\n\nThe config:\n\n    config :my_app, MyApp.ClusteredCache,\n      mode: :client_side_cluster,\n      client_side_cluster: [\n        nodes: [\n          node1: [\n            pool_size: 10,\n            conn_opts: [\n              host: \"127.0.0.1\",\n              port: 9001\n            ]\n          ],\n          node2: [\n            pool_size: 4,\n            conn_opts: [\n              url: \"redis://127.0.0.1:9002\"\n            ]\n          ],\n          node3: [\n            conn_opts: [\n              host: \"127.0.0.1\",\n              port: 9003\n            ]\n          ],\n          ...\n        ]\n      ]","ref":"NebulexRedisAdapter.html#module-client-side-cluster","title":"Client-side Cluster - NebulexRedisAdapter","type":"module"},{"doc":"In addition to `Nebulex.Cache` config options, the adapter supports the\nfollowing options:\n\n* `:mode` - Redis configuration mode.\n\n    * `:standalone` - A single Redis instance. See the\n      [\"Standalone\"](#module-standalone) section in the\n      module documentation for more options.\n    * `:redis_cluster` - Redis Cluster setup. See the\n      [\"Redis Cluster\"](#module-redis-cluster) section in the\n      module documentation for more options.\n    * `:client_side_cluster` - See the\n      [\"Client-side Cluster\"](#module-client-side-cluster) section in the\n      module documentation for more options.\n\n  The default value is `:standalone`.\n\n* `:pool_size` (`t:pos_integer/0`) - The number of connections that will be started by the adapter\n  (based on the `:mode`). The default value is `System.schedulers_online()`.\n\n* `:serializer` - Custom serializer module implementing the `NebulexRedisAdapter.Serializer`\n  behaviour.\n\n* `:serializer_opts` (`t:keyword/0`) - Custom serializer options. The default value is `[]`.\n\n  * `:encode_key` (`t:keyword/0`) - Options for encoding the key. The default value is `[]`.\n\n  * `:encode_value` (`t:keyword/0`) - Options for encoding the value. The default value is `[]`.\n\n  * `:decode_key` (`t:keyword/0`) - Options for decoding the key. The default value is `[]`.\n\n  * `:decode_value` (`t:keyword/0`) - Options for decoding the value. The default value is `[]`.\n\n* `:conn_opts` (`t:keyword/0`) - Redis client options. See `Redix` docs for more information\n  about the options. The default value is `[host: \"127.0.0.1\", port: 6379]`.\n\n* `:redis_cluster` - Required only when `:mode` is set to `:redis_cluster`. A keyword list of\n  options.\n\n  See [\"Redis Cluster options\"](#module-redis-cluster-options)\n  section below.\n\n* `:client_side_cluster` - Required only when `:mode` is set to `:client_side_cluster`. A keyword\n  list of options.\n\n  See [\"Client-side Cluster options\"](#module-client-side-cluster-options)\n  section below.\n\n#","ref":"NebulexRedisAdapter.html#module-configuration-options","title":"Configuration options - NebulexRedisAdapter","type":"module"},{"doc":"The available options are:\n\n* `:configuration_endpoints` - Required. A keyword list of named endpoints where the key is an atom as\n  an identifier and the value is another keyword list of options\n  (same as `:conn_opts`).\n\n  See [\"Redis Cluster\"](#module-redis-cluster) for more information.\n\n* `:override_master_host` (`t:boolean/0`) - Defines whether the given master host should be overridden with the\n  configuration endpoint or not. Defaults to `false`.\n\n  The adapter uses the host returned by the **\"CLUSTER SHARDS\"**\n  (Redis >= 7) or **\"CLUSTER SLOTS\"** (Redis < 7) command. One may\n  consider set it to `true` for tests when using Docker for example,\n  or when Redis nodes are behind a load balancer that Redis doesn't\n  know the endpoint of. See Redis docs for more information.\n\n  The default value is `false`.\n\n* `:keyslot` - The module implementing the `Nebulex.Adapter.Keyslot`\n  behaviour, which is used to compute the node where the command\n  will be applied to. The default value is `NebulexRedisAdapter.RedisCluster.Keyslot`.\n\n#","ref":"NebulexRedisAdapter.html#module-redis-cluster-options","title":"Redis Cluster options - NebulexRedisAdapter","type":"module"},{"doc":"The available options are:\n\n* `:nodes` - Required. A keyword list of named nodes where the key is an atom as\n  an identifier and the value is another keyword list of options\n  (same as `:conn_opts`).\n\n  See [\"Client-side Cluster\"](#module-client-side-cluster)\n  for more information.\n\n* `:keyslot` - The module implementing the `Nebulex.Adapter.Keyslot`\n  behaviour, which is used to compute the node where the command\n  will be applied to. The default value is `NebulexRedisAdapter.ClientCluster.Keyslot`.","ref":"NebulexRedisAdapter.html#module-client-side-cluster-options","title":"Client-side Cluster options - NebulexRedisAdapter","type":"module"},{"doc":"Since the adapter runs on top of `Redix`, all commands accept their options\n(e.g.: `:timeout`, and `:telemetry_metadata`). See `Redix` docs for more\ninformation.\n\n#","ref":"NebulexRedisAdapter.html#module-shared-runtime-options","title":"Shared runtime options - NebulexRedisAdapter","type":"module"},{"doc":"The following options apply to all commands:\n\n  * `:lock_retries` - This option is specific to the `:redis_cluster` mode.\n    When the config manager is running and setting up the hash slot map,\n    all Redis commands get blocked until the cluster is properly configured\n    and the hash slot map is ready to use. This option defines the max retry\n    attempts to acquire the lock and execute the command in case the\n    configuration manager is running and all commands are locked.\n    Defaults to `:infinity`.","ref":"NebulexRedisAdapter.html#module-redis-cluster-runtime-options","title":"Redis Cluster runtime options - NebulexRedisAdapter","type":"module"},{"doc":"As it is mentioned in the configuration options above, the `:redis_cluster`\nand `:client_side_cluster` modes have a default value for the `:keyslot`\noption. However, you can also provide your own implementation by implementing\nthe `Nebulex.Adapter.Keyslot` and configuring the `:keyslot` option.\nFor example:\n\n    defmodule MyApp.ClusteredCache.Keyslot do\n      use Nebulex.Adapter.Keyslot\n\n      @impl true\n      def hash_slot(key, range) do\n        # your implementation goes here\n      end\n    end\n\nAnd the config:\n\n    config :my_app, MyApp.ClusteredCache,\n      mode: :client_side_cluster,\n      client_side_cluster: [\n        keyslot: MyApp.ClusteredCache.Keyslot,\n        nodes: [\n          ...\n        ]\n      ]\n\n> **NOTE:** For `:redis_cluster` mode, the`:keyslot` implementation must\n  follow the [Redis cluster specification][redis_cluster_spec].\n\n[redis_cluster_spec]: https://redis.io/docs/reference/cluster-spec/","ref":"NebulexRedisAdapter.html#module-custom-keyslot","title":"Custom Keyslot - NebulexRedisAdapter","type":"module"},{"doc":"As is explained in `Nebulex.Cache`, most of the write-like functions support\nthe `:ttl` option to define the expiration time, and it is defined in\n**milliseconds**. Despite Redis work with **seconds**, the conversion logic\nis handled by the adapter transparently, so when using a cache even with the\nRedis adapter, be sure you pass the `:ttl` option in **milliseconds**.","ref":"NebulexRedisAdapter.html#module-ttl-or-expiration-time","title":"TTL or Expiration Time - NebulexRedisAdapter","type":"module"},{"doc":"Currently. the adapter only works with strings, which means a given Elixir\nterm is encoded to a binary/string before executing a command. Similarly,\na returned binary from Redis after executing a command is decoded into an\nElixir term. The encoding/decoding process is performed by the adapter\nunder-the-hood. However, it is possible to provide a custom serializer via the\noption `:serializer`. The value must be module implementing the\n`NebulexRedisAdapter.Serializer` behaviour.\n\n**NOTE:** Support for other Redis Data Types is in the roadmap.","ref":"NebulexRedisAdapter.html#module-data-types","title":"Data Types - NebulexRedisAdapter","type":"module"},{"doc":"Since the queryable API is implemented by using `KEYS` command,\nkeep in mind the following caveats:\n\n  * Only keys can be queried.\n  * Only strings and predefined queries are allowed as query values.\n\n#","ref":"NebulexRedisAdapter.html#module-queryable-api","title":"Queryable API - NebulexRedisAdapter","type":"module"},{"doc":"* `nil` - All keys are returned.\n\n  * `{:in, [term]}` - Only the keys in the given key list (`[term]`)\n    are returned. This predefined query is only supported for\n    `c:Nebulex.Cache.delete_all/2`. This is the recommended\n    way of doing bulk delete of keys.\n\n#","ref":"NebulexRedisAdapter.html#module-predefined-queries","title":"Predefined queries - NebulexRedisAdapter","type":"module"},{"doc":"iex> MyApp.RedisCache.put_all(%{\n    ...>   \"firstname\" => \"Albert\",\n    ...>   \"lastname\" => \"Einstein\",\n    ...>   \"age\" => 76\n    ...> })\n    :ok\n\n    iex> MyApp.RedisCache.all(\"**name**\")\n    [\"firstname\", \"lastname\"]\n\n    iex> MyApp.RedisCache.all(\"a??\")\n    [\"age\"]\n\n    iex> MyApp.RedisCache.all()\n    [\"age\", \"firstname\", \"lastname\"]\n\n    iex> stream = MyApp.RedisCache.stream(\"**name**\")\n    iex> stream |> Enum.to_list()\n    [\"firstname\", \"lastname\"]\n\n    # get the values for the returned queried keys\n    iex> \"**name**\" |> MyApp.RedisCache.all() |> MyApp.RedisCache.get_all()\n    %{\"firstname\" => \"Albert\", \"lastname\" => \"Einstein\"}\n\n#","ref":"NebulexRedisAdapter.html#module-examples","title":"Examples - NebulexRedisAdapter","type":"module"},{"doc":"iex> MyApp.RedisCache.delete_all({:in, [\"foo\", \"bar\"]})\n    2","ref":"NebulexRedisAdapter.html#module-deleting-multiple-keys-at-once-bulk-delete","title":"Deleting multiple keys at once (bulk delete) - NebulexRedisAdapter","type":"module"},{"doc":"This adapter doesn't provide support for transactions. However, in the future,\nit is planned support [Redis Transactions][redis_transactions] by using the\ncommands `MULTI`, `EXEC`, `DISCARD` and `WATCH`.\n\n[redis_transactions]: https://redis.io/docs/manual/transactions/","ref":"NebulexRedisAdapter.html#module-transactions","title":"Transactions - NebulexRedisAdapter","type":"module"},{"doc":"Since `NebulexRedisAdapter` works on top of `Redix` and provides features like\nconnection pools and \"Redis Cluster\" support, it may be seen also as a sort of\nRedis client, but it is meant to be used mainly with the Nebulex cache API.\nHowever, Redis API is quite extensive and there are a lot of useful commands\nwe may want to run taking advantage of the `NebulexRedisAdapter` features.\nTherefore, the adapter provides two additional/extended functions to the\ndefined cache: `command!/2` and `pipeline!/2`.\n\n### `command!(command, opts \\\\ [])`\n\n    iex> MyCache.command!([\"LPUSH\", \"mylist\", \"world\"], key: \"mylist\")\n    1\n    iex> MyCache.command!([\"LPUSH\", \"mylist\", \"hello\"], key: \"mylist\")\n    2\n    iex> MyCache.command!([\"LRANGE\", \"mylist\", \"0\", \"-1\"], key: \"mylist\")\n    [\"hello\", \"world\"]\n\n### `pipeline!(commands, opts \\\\ [])`\n\n    iex> [\n    ...>   [\"LPUSH\", \"mylist\", \"world\"],\n    ...>   [\"LPUSH\", \"mylist\", \"hello\"],\n    ...>   [\"LRANGE\", \"mylist\", \"0\", \"-1\"]\n    ...> ]\n    ...> |> cache.pipeline!(key: \"mylist\")\n    [1, 2, [\"hello\", \"world\"]]\n\n#","ref":"NebulexRedisAdapter.html#module-running-redis-commands-and-or-pipelines","title":"Running Redis commands and/or pipelines - NebulexRedisAdapter","type":"module"},{"doc":"* `:key` - It is required when used the adapter in mode `:redis_cluster`\n    or `:client_side_cluster` so that the node where the commands will\n    take place can be selected properly. For `:standalone` mode is not\n    required (optional).\n  * `:name` - The name of the cache in case you are using dynamic caches,\n    otherwise it is not required.\n\nSince these functions run on top of `Redix`, they also accept their options\n(e.g.: `:timeout`, and `:telemetry_metadata`). See `Redix` docs for more\ninformation.","ref":"NebulexRedisAdapter.html#module-options-for-command-2-and-pipeline-2","title":"Options for `command!/2` and `pipeline!/2`: - NebulexRedisAdapter","type":"module"},{"doc":"This adapter emits the recommended Telemetry events.\nSee the \"Telemetry events\" section in `Nebulex.Cache`\nfor more information.\n\n#","ref":"NebulexRedisAdapter.html#module-telemetry","title":"Telemetry - NebulexRedisAdapter","type":"module"},{"doc":"Aside from the recommended Telemetry events by `Nebulex.Cache`, this adapter\nexposes the following Telemetry events for the `:redis_cluster` mode:\n\n  * `telemetry_prefix ++ [:config_manager, :setup, :start]` - This event is\n    specific to the `:redis_cluster` mode. Before the configuration manager\n    calls Redis to set up the cluster shards, this event should be invoked.\n\n    The `:measurements` map will include the following:\n\n    * `:system_time` - The current system time in native units from calling:\n      `System.system_time()`.\n\n    A Telemetry `:metadata` map including the following fields:\n\n    * `:adapter_meta` - The adapter metadata.\n    * `:pid` - The configuration manager PID.\n\n  * `telemetry_prefix ++ [:config_manager, :setup, :stop]` - This event is\n    specific to the `:redis_cluster` mode. After the configuration manager\n    set up the cluster shards, this event should be invoked.\n\n    The `:measurements` map will include the following:\n\n    * `:duration` - The time spent configuring the cluster. The measurement\n      is given in the `:native` time unit. You can read more about it in the\n      docs for `System.convert_time_unit/3`.\n\n    A Telemetry `:metadata` map including the following fields:\n\n    * `:adapter_meta` - The adapter metadata.\n    * `:pid` - The configuration manager PID.\n    * `:status` - The cluster setup status. If the cluster was configured\n      successfully, the status will be set to `:ok`, otherwise, will be\n      set to `:error`.\n    * `:error` - The status reason. When the status is `:ok`, the reason is\n      `:succeeded`, otherwise, it is the error reason.\n\n  * `telemetry_prefix ++ [:config_manager, :setup, :exception]` - This event\n    is specific to the `:redis_cluster` mode. When an exception is raised\n    while configuring the cluster, this event should be invoked.\n\n    The `:measurements` map will include the following:\n\n    * `:duration` - The time spent configuring the cluster. The measurement\n      is given in the `:native` time unit. You can read more about it in the\n      docs for `System.convert_time_unit/3`.\n\n    A Telemetry `:metadata` map including the following fields:\n\n    * `:adapter_meta` - The adapter metadata.\n    * `:pid` - The configuration manager PID.\n    * `:kind` - The type of the error: `:error`, `:exit`, or `:throw`.\n    * `:reason` - The reason of the error.\n    * `:stacktrace` - The stacktrace.","ref":"NebulexRedisAdapter.html#module-adapter-specific-telemetry-events-for-the-redis_cluster-mode","title":"Adapter-specific telemetry events for the `:redis_cluster` mode - NebulexRedisAdapter","type":"module"},{"doc":"This server takes care of initialization and cleanup jobs. For example,\nattaching the stats handler when the cache starts and detaching it when\nit terminates.","ref":"NebulexRedisAdapter.BootstrapServer.html","title":"NebulexRedisAdapter.BootstrapServer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NebulexRedisAdapter.BootstrapServer.html#child_spec/1","title":"NebulexRedisAdapter.BootstrapServer.child_spec/1","type":"function"},{"doc":"","ref":"NebulexRedisAdapter.BootstrapServer.html#start_link/1","title":"NebulexRedisAdapter.BootstrapServer.start_link/1","type":"function"},{"doc":"Default Keyslot implementation when `:jchash` module is loaded.","ref":"NebulexRedisAdapter.ClientCluster.Keyslot.html","title":"NebulexRedisAdapter.ClientCluster.Keyslot","type":"module"},{"doc":"Default `Nebulex.Adapter.Keyslot` implementation.","ref":"NebulexRedisAdapter.RedisCluster.Keyslot.html","title":"NebulexRedisAdapter.RedisCluster.Keyslot","type":"module"},{"doc":"Redis Cluster Node/Slot Supervisor.","ref":"NebulexRedisAdapter.RedisCluster.PoolSupervisor.html","title":"NebulexRedisAdapter.RedisCluster.PoolSupervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NebulexRedisAdapter.RedisCluster.PoolSupervisor.html#child_spec/1","title":"NebulexRedisAdapter.RedisCluster.PoolSupervisor.child_spec/1","type":"function"},{"doc":"A **Serializer** encodes keys and values sent to Redis,\nand decodes keys and values in the command output.\n\nSee [Redis Strings](https://redis.io/docs/data-types/strings/).","ref":"NebulexRedisAdapter.Serializer.html","title":"NebulexRedisAdapter.Serializer","type":"behaviour"},{"doc":"Decodes `key` with the given `opts`.","ref":"NebulexRedisAdapter.Serializer.html#c:decode_key/2","title":"NebulexRedisAdapter.Serializer.decode_key/2","type":"callback"},{"doc":"Decodes `value` with the given `opts`.","ref":"NebulexRedisAdapter.Serializer.html#c:decode_value/2","title":"NebulexRedisAdapter.Serializer.decode_value/2","type":"callback"},{"doc":"Encodes `key` with the given `opts`.","ref":"NebulexRedisAdapter.Serializer.html#c:encode_key/2","title":"NebulexRedisAdapter.Serializer.encode_key/2","type":"callback"},{"doc":"Encodes `value` with the given `opts`.","ref":"NebulexRedisAdapter.Serializer.html#c:encode_value/2","title":"NebulexRedisAdapter.Serializer.encode_value/2","type":"callback"},{"doc":"Protocol controlling how a key/value is encoded to a string\nand how a string is decoded into an Elixir term.\n\nSee [Redis Strings](https://redis.io/docs/data-types/strings/).","ref":"NebulexRedisAdapter.Serializer.Serializable.html","title":"NebulexRedisAdapter.Serializer.Serializable","type":"protocol"},{"doc":"Decodes `data` with the given `opts`.","ref":"NebulexRedisAdapter.Serializer.Serializable.html#decode/2","title":"NebulexRedisAdapter.Serializer.Serializable.decode/2","type":"function"},{"doc":"Encodes `data` with the given `opts`.","ref":"NebulexRedisAdapter.Serializer.Serializable.html#encode/2","title":"NebulexRedisAdapter.Serializer.Serializable.encode/2","type":"function"},{"doc":"","ref":"NebulexRedisAdapter.Serializer.Serializable.html#t:t/0","title":"NebulexRedisAdapter.Serializer.Serializable.t/0","type":"type"},{"doc":"NebulexRedisAdapter error.","ref":"NebulexRedisAdapter.Error.html","title":"NebulexRedisAdapter.Error","type":"exception"},{"doc":"","ref":"NebulexRedisAdapter.Error.html#format_error/2","title":"NebulexRedisAdapter.Error.format_error/2","type":"function"},{"doc":"Error reason type","ref":"NebulexRedisAdapter.Error.html#t:reason/0","title":"NebulexRedisAdapter.Error.reason/0","type":"type"},{"doc":"Error type","ref":"NebulexRedisAdapter.Error.html#t:t/0","title":"NebulexRedisAdapter.Error.t/0","type":"type"}]}